/*********************************************
 * OPL 12.4 Model
 * Author: mwatson, pcacioppi
 * Creation Date: Jan 2013 
 * 
 * This model is a simplification of the multi-product model from chapter 10 of SNS
 *********************************************/


{string} Plants =...;  //this is L in the book and called a plant or supplier
{string} Warehouses =...;  //this is I in the book and called a facility
{string} Customers =...;  // this is J in the book
{string} Products =...;  // this is K in the book
int MaxWarehousesP =...;  // this is the number of warehouses to locate
float Demand[Customers][Products] =...;  // this is d in the book
float PlantToWarehouseShippingCost[Plants][Warehouses][Products]=...;  
		// this is transPW in the book
float WarehouseToCustomerShippingCost[Warehouses][Customers][Products]=...;  
		// this is transWC in the book
float WarehouseFixedCost[Warehouses]=...;  
		// this is the fixed cost of opening a warehouse
float WarehouseCapacity[Warehouses]=...;  
		// this is cross product capacity available at a warehouse
float DemandVolume[Warehouses][Customers][Products] = ...; 
		// this is the warehouse volume of a demand entry
float PlantProductionCapacity[Plants][Products] = ...; 
		// this is the production capacity of a plant for a given product
dvar boolean OpenVar[Warehouses];  // this is X in the book
dvar boolean AssignVar[Warehouses][Customers][Products];  
		//this is Y in the book. Assigns a demand point to a warehouse
dvar float+ PlantToWarehouseShippingVar[Plants][Warehouses][Products]; 
		// this is the shipment variable for plants to warehouses
 
 // we don't use metric variables here, but in practice we would.
 minimize
  sum (p in Plants, w in Warehouses, k in Products)
	PlantToWarehouseShippingCost[p][w][k] * PlantToWarehouseShippingVar[p][w][k] +
  sum(w in Warehouses, c in Customers, k in Products)
    WarehouseToCustomerShippingCost[w][c][k]*Demand[c][k]*AssignVar[w][c][k] + 
  sum(w in Warehouses)
  	  WarehouseFixedCost[w] * OpenVar[w];
    
    
subject to{

  forall ( c in Customers, k in Products )
    ctEachCustomersDemandMustBeMet:
    	sum( w in Warehouses )
    	  AssignVar[w][c][k]==1;	
  
    ctOnlyUse_P_Warehouses:
  	sum(w in Warehouses)
  	  OpenVar[w]==MaxWarehousesP;
  
    forall (w in Warehouses, c in Customers, k in Products)
    ctCannotAssignCustomertoWH_UnlessItIsOpen:
      AssignVar[w][c][k] <= OpenVar[w];
    
    forall (w in Warehouses)
    ctWarehouseCapacityConstraints:
      sum(c in Customers, k in Products)
        Demand[c][k]*AssignVar[w][c][k]*DemandVolume[w][c][k]<= WarehouseCapacity[w]*OpenVar[w] ;
        
    forall (w in Warehouses, k in Products)
    ctConservationOfFlow:
		sum(p in Plants) PlantToWarehouseShippingVar[p][w][k] ==  sum(c in Customers)Demand[c][k]*AssignVar[w][c][k];
    
    forall (p in Plants, k in Products)
		ctPlantProductionCapacity:
		sum(w in Warehouses)  PlantToWarehouseShippingVar[p][w][k] <= PlantProductionCapacity[p][k];
        
}