/*********************************************
 * OPL 12.4 Model
 * Author: mwatson, pcacioppi
 * Creation Date: Jan 2013 
 * 
 * This model is adjusts the simplified multi-product model to exploit data sparsity
 *********************************************/


{string} Plants =...;  //this is L in the book and called a plant or supplier
{string} Warehouses =...;  //this is I in the book and called a facility
{string} Customers =...;  // this is J in the book
{string} Products =...;  // this is K in the book
int MaxWarehousesP =...;  // this is the number of warehouses to locate

/* create data tuples and their associated data sets */
tuple plantToWarehouseLane
{	string plant;
	string warehouse;
	string product;  }
{plantToWarehouseLane} PlantToWarehouseLanes = ...;

tuple warehouseToCustomerLane
{	string warehouse;
	string customer;
	string product;   }
{warehouseToCustomerLane} WarehouseToCustomerLanes = ...;

tuple demandRecord
{ 	string customer;
	string product;   }
{demandRecord} DemandRecords = ...;

tuple supplyRecord
{	string plant;
	string product;  }
{supplyRecord} SupplyRecords = ...;

float Demand[DemandRecords] =...;  // this is d in the book
float PlantToWarehouseShippingCost[PlantToWarehouseLanes]=...;  // this is transPW in the book
float WarehouseToCustomerShippingCost[WarehouseToCustomerLanes]=...;  // this is transWC in the book
float WarehouseFixedCost[Warehouses]=...;  // this is the fixed cost of opening a warehouse
float WarehouseCapacity[Warehouses]=...;  // this is cross product capacity available at a warehouse
float DemandVolume[WarehouseToCustomerLanes] = ...; // this is the warehouse volume of a demand entry
float PlantProductionCapacity[SupplyRecords] = ...; // this is the production capacity of a plant for a given product
dvar boolean OpenVar[Warehouses];  // this is X in the book
dvar boolean AssignVar[WarehouseToCustomerLanes];  //this is Y in the book. Assigns a demand point to a warehouse
dvar float+ PlantToWarehouseShippingVar[PlantToWarehouseLanes]; // this is the shipment variable for plants to warehouses
 

 minimize
  sum (l in PlantToWarehouseLanes)
	PlantToWarehouseShippingCost[l] * PlantToWarehouseShippingVar[l] +
  sum(<s, d, k> in WarehouseToCustomerLanes)
    WarehouseToCustomerShippingCost[<s, d, k>]*Demand[<d, k>]*AssignVar[<s, d, k>] + 
  sum(w in Warehouses)
  	  WarehouseFixedCost[w] * OpenVar[w];
    
subject to{

  forall ( <c, k> in DemandRecords )
    ctEachCustomersDemandMustBeMet:
    	sum( <w, c, k> in WarehouseToCustomerLanes )
    	  AssignVar[<w, c, k>]==1;	
  
    ctOnlyUse_P_Warehouses:
  	sum(w in Warehouses)
  	  OpenVar[w]==MaxWarehousesP;
  
    forall (<w, c, k> in WarehouseToCustomerLanes)
    ctCannotAssignCustomertoWH_UnlessItIsOpen:
      AssignVar[<w, c, k>] <= OpenVar[w];
    
    forall (w in Warehouses)
    ctWarehouseCapacityConstraints:
      sum(<w, c, k> in WarehouseToCustomerLanes) 
        Demand[<c, k>]*AssignVar[<w, c, k>]*DemandVolume[<w, c, k>] <= WarehouseCapacity[w]*OpenVar[w] ;
        
    forall (w in Warehouses, k in Products)
    ctConservationOfFlow:
		sum(<p, w, k> in PlantToWarehouseLanes) PlantToWarehouseShippingVar[<p, w, k>] ==  
				sum(<w, c, k> in WarehouseToCustomerLanes)Demand[<c, k>]*AssignVar[<w, c, k>];
    
    forall (<p, k> in SupplyRecords)
		ctPlantProductionCapacity:
		sum(<p, w, k> in PlantToWarehouseLanes)  
			PlantToWarehouseShippingVar[<p, w, k>] <= PlantProductionCapacity[<p, k>];
}